package com.ekexiu.project.bridge.resource.service;

import com.ekexiu.project.bridge.resource.dao.BridgeDao;
import com.ekexiu.project.bridge.resource.dao.BridgeServerDao;
import com.ekexiu.project.bridge.resource.dao.CollectDeviceDao;
import com.ekexiu.project.bridge.resource.dao.TransducerDao;
import com.ekexiu.project.bridge.resource.po.Bridge;
import com.ekexiu.project.bridge.resource.po.BridgeServer;
import com.ekexiu.project.bridge.resource.po.CollectDevice;
import com.ekexiu.project.bridge.resource.po.Transducer;
import com.ekexiu.project.bridge.system.vo.SessionUser;
import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.web.annotation.LoginUser;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.apt.web.annotation.param.Upload;
import org.jfw.util.PageQueryResult;
import org.jfw.util.StringUtil;
import org.jfw.util.web.fileupload.Item;
import org.jfw.util.web.fileupload.UploadItemIterator;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * Created by TT on 2018/8/8.
 */
@Path
public class BridgeService {

    private static final AtomicInteger FN_IDX = new AtomicInteger(1);

    private File imgPath;

    @Autowrie
    private BridgeDao bridgeDao;
    @Autowrie
    private BridgeServerDao bridgeServerDao;
    @Autowrie
    private CollectDeviceDao collectDeviceDao;
    @Autowrie
    private TransducerDao transducerDao;

    public BridgeDao getBridgeDao() {
        return bridgeDao;
    }

    public void setBridgeDao(BridgeDao bridgeDao) {
        this.bridgeDao = bridgeDao;
    }

    public BridgeServerDao getBridgeServerDao() {
        return bridgeServerDao;
    }

    public void setBridgeServerDao(BridgeServerDao bridgeServerDao) {
        this.bridgeServerDao = bridgeServerDao;
    }

    public CollectDeviceDao getCollectDeviceDao() {
        return collectDeviceDao;
    }

    public void setCollectDeviceDao(CollectDeviceDao collectDeviceDao) {
        this.collectDeviceDao = collectDeviceDao;
    }

    public TransducerDao getTransducerDao() {
        return transducerDao;
    }

    public void setTransducerDao(TransducerDao transducerDao) {
        this.transducerDao = transducerDao;
    }

    public File getImgPath() {
        return imgPath;
    }

    public void setImgPath(File imgPath) {
        this.imgPath = imgPath;
    }

    /**
     * 上传文件
     *
     * @param it
     * @return
     * @throws Exception
     */
    @Path("/bridge/upload")
    @Post
    public List<UploadFile> upload(@Upload UploadItemIterator it) throws Exception {
        List<UploadFile> ret = new LinkedList<UploadFile>();
        try {
            while (it.hasNext()) {
                Item item = it.next();
                if (!item.isFormField()) {
                    String name = normalizeFileName(item.getName());
                    int index = name.lastIndexOf('.');
                    String ext = index >= 0 ? name.substring(index + 1) : "";
                    ext = ext.trim();
                    long fileLen = 0;
                    int len = 0;
                    UploadFile uf = this.buildTargetFile(ext);
                    uf.setName(name);
                    File file = uf.getFn();
                    ret.add(uf);
                    byte[] buf = new byte[8092];
                    OutputStream of = new FileOutputStream(file);
                    try {
                        InputStream in = item.getInputStream();
                        try {
                            while ((len = in.read(buf)) >= 0) {
                                if (len > 0) {
                                    of.write(buf, 0, len);
                                    fileLen += len;
                                }
                            }
                            uf.setSize(fileLen);
                        } finally {
                            in.close();
                        }
                    } finally {
                        of.close();
                    }
                }
            }
            return ret;
        } catch (Exception e) {
            this.remove(ret);
            throw e;
        } finally {
            if (it != null)
                it.clean();
        }
    }



    @Post
    @Path("/bridge")
    public String insert(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, Bridge bridge) throws SQLException {
        String id = StringUtil.buildUUID();
        bridge.setId(id);
        bridge.setActive(true);
        bridge.setCreator(sessionUser.getId());
        bridge.setModifier(sessionUser.getId());
        this.bridgeDao.insert(con, bridge);
        return id;
    }

    @Get
    @Path("/bridge/byUser")
    public PageQueryResult<Bridge> query(@JdbcConn Connection con, @LoginUser SessionUser user, @Nullable Boolean active, int pageSize, int pageNo) throws SQLException {
        return this.bridgeDao.pageQuery(con, active, user.getId(), pageSize, pageNo);
    }

    @Get
    @Path("/bridge/qo")
    public Bridge queryBridge(@JdbcConn Connection con, String id) throws SQLException {
        return this.bridgeDao.query(con, id);
    }

    @Get
    @Path("/bridge/byCode")
    public Bridge queryByCode(@JdbcConn Connection con,String code)throws SQLException {
        return this.bridgeDao.queryByCode(con, code);
    }

    @Get
    @Path("/bridge/checkCode")
    public Bridge queryByCode(@JdbcConn Connection con,String code,@Nullable String id,@Nullable Boolean active)throws SQLException {
        return this.bridgeDao.queryByCode(con, code, id,active);
    }
    @Get
    @Path("/bridge/delete")
    public void delete(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, String id) throws SQLException {
        this.bridgeDao.logicDelete(con, sessionUser.getId(), id);
        this.bridgeServerDao.logicDeleteByBridge(con, sessionUser.getId(), id);
        String[] bridgeId = new String[1];
        bridgeId[0] = id;
        List<BridgeServer> bridgeServers = this.bridgeServerDao.queryByBridges(con, null, bridgeId);
        if (bridgeServers.size() > 0) {
            String[] serverId = new String[bridgeServers.size()];
            for (int i = 0; i < bridgeServers.size(); ++i) {
                serverId[i] = bridgeServers.get(i).getId();
            }
            this.collectDeviceDao.logicDeleteByServers(con, sessionUser.getId(), serverId);
            List<CollectDevice> collectDevices = this.collectDeviceDao.queryByServers(con, null, serverId);
            if (collectDevices.size() > 0) {
                String[] deviceId = new String[collectDevices.size()];
                for (int i = 0; i < collectDevices.size(); ++i) {
                    deviceId[i] = collectDevices.get(i).getId();
                }
                this.transducerDao.logicDeleteByDevices(con, null, deviceId);
            }
        }
    }

    @Post
    @Path("/bridge/update")
    public void update(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, Bridge bridge) throws SQLException {
        bridge.setModifier(sessionUser.getId());
        this.bridgeDao.update(con, bridge);
    }

    @Get
    @Path("/bridge/pq")
    public PageQueryResult<Bridge> pageQuery(@JdbcConn Connection con, @Nullable Boolean active, @Nullable String name, @Nullable String code, int pageSize, int pageNo) throws SQLException {
        return this.bridgeDao.pageQuery(con, active, name == null ? null : "%" + name + "%", code, pageSize, pageNo);
    }

    @Get
    @Path("/bridge/list")
    public List<Bridge> listQuery(@JdbcConn Connection con,@Nullable Boolean active,@Nullable String key)throws SQLException {
        return this.bridgeDao.listQueryByKey(con,active, key == null ? null : "%" + key + "%");
    }

    @Get
    @Path("/bridge/byUserId")
    public List<Bridge> queryByUserId(@JdbcConn Connection con,@Nullable Boolean active,String uid)throws SQLException {
        return this.bridgeDao.listQueryByUser(con, active, uid);
    }

    @Get
    @Path("/bridge/checkName")
    public Bridge queryByName(@JdbcConn Connection con, String name,@Nullable String id,@Nullable Boolean active)throws SQLException {
        return this.bridgeDao.queryByName(con, name, id,active);
    }

    @Get
    @Path("/bridge/checkShortName")
    public Bridge queryByShortName(@JdbcConn Connection con,String shortName,@Nullable String id,@Nullable Boolean active)throws SQLException {
        return this.bridgeDao.queryByShortName(con, shortName, id,active);
    }

    @Post
    @Path("/server")
    public String insert(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, BridgeServer bridgeServer) throws SQLException {
        String id = StringUtil.buildUUID();
        bridgeServer.setId(id);
        bridgeServer.setActive(true);
        bridgeServer.setCreator(sessionUser.getId());
        bridgeServer.setModifier(sessionUser.getId());
        this.bridgeServerDao.insert(con, bridgeServer);
        return id;
    }

    @Get
    @Path("/server/qo")
    public BridgeServer queryServer(@JdbcConn Connection con, String id) throws SQLException {
        return this.bridgeServerDao.query(con, id);
    }

    @Get
    @Path("/server/pq")
    public PageQueryResult<BridgeServer> pageQueryServer(@JdbcConn Connection con, @Nullable Boolean active, @Nullable String bcode, @Nullable String code, int pageSize, int pageNo) throws SQLException {
        return this.bridgeServerDao.pageQuery(con, active, bcode, code, pageSize, pageNo);
    }

    @Post
    @Path("/server/update")
    public void updateServer(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, BridgeServer bridgeServer) throws SQLException {
        bridgeServer.setModifier(sessionUser.getId());
        this.bridgeServerDao.update(con, bridgeServer);
    }

    @Get
    @Path("/server/byBridge")
    public List<BridgeServer> queryByBridge(@JdbcConn Connection con,@Nullable Boolean active, String id)throws SQLException {
        return this.bridgeServerDao.queryByBridge(con,active, id);
    }

    @Get
    @Path("/server/delete")
    public void deleteServer(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser
            , String id) throws SQLException {
        this.bridgeServerDao.logicDelete(con, sessionUser.getId(), id);
        String[] serverId = new String[1];
        serverId[0] = id;
        this.collectDeviceDao.logicDeleteByServers(con, sessionUser.getId(), serverId);
        List<CollectDevice> collectDevices = this.collectDeviceDao.queryByServers(con, null, serverId);
        if (collectDevices.size() > 0) {
            String[] deviceId = new String[collectDevices.size()];
            for (int i = 0; i < collectDevices.size(); ++i) {
                deviceId[i] = collectDevices.get(i).getId();
            }
            this.transducerDao.logicDeleteByDevices(con, null, deviceId);
        }
    }

    @Get
    @Path("/server/listByCode")
    public List<BridgeServer> serverListByCode(@JdbcConn Connection con,@Nullable Boolean active,@Nullable String code)throws SQLException {
        return this.bridgeServerDao.queryByCode(con, active, code == null ? null : "%" + code + "%");
    }


    @Get
    @Path("/server/checkSeq")
    public BridgeServer serverBySeq(@JdbcConn Connection con,String seq,@Nullable String id,@Nullable Boolean active)throws SQLException {
        return this.bridgeServerDao.checkBySeq(con, seq,id,active);
    }

    @Get
    @Path("/server/checkCode")
    public BridgeServer serverByCode(@JdbcConn Connection con,String code,@Nullable String id,@Nullable Boolean active)throws SQLException {
        return this.bridgeServerDao.queryByCode(con, code, id,active);
    }

    @Post
    @Path("/device")
    public String insert(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, CollectDevice collectDevice) throws SQLException {
        String id = StringUtil.buildUUID();
        collectDevice.setId(id);
        collectDevice.setActive(true);
        collectDevice.setCreator(sessionUser.getId());
        collectDevice.setModifier(sessionUser.getId());
        this.collectDeviceDao.insert(con, collectDevice);
        return id;
    }

    @Get
    @Path("/device/qo")
    public CollectDevice queryDevice(@JdbcConn Connection con, String id) throws SQLException {
        return this.collectDeviceDao.query(con, id);
    }

    @Get
    @Path("/device/pq")
    public PageQueryResult<CollectDevice> pageQueryDevice(@JdbcConn Connection con, @Nullable Boolean active, @Nullable String scode, @Nullable String code, int pageSize, int pageNo) throws SQLException {
        return this.collectDeviceDao.pageQuery(con, active, scode, code, pageSize, pageNo);
    }

    @Post
    @Path("/device/update")
    public void updateDevice(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, CollectDevice collectDevice) throws SQLException {
        collectDevice.setModifier(sessionUser.getId());
        this.collectDeviceDao.update(con, collectDevice);
    }

    @Get
    @Path("/device/delete")
    public void deleteDevice(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, String id) throws SQLException {
        this.collectDeviceDao.logicDelete(con, sessionUser.getId(), id);
        String[] deviceId = new String[1];
        deviceId[0] = id;
        this.transducerDao.logicDeleteByDevices(con, null, deviceId);
    }

    @Get
    @Path("/device/byServer")
    public List<CollectDevice> queryByServer(@JdbcConn Connection con,@Nullable Boolean active,String id)throws SQLException {
        return this.collectDeviceDao.queryByServer(con,active, id);
    }

    @Get
    @Path("/device/listByCode")
    public List<CollectDevice> deviceListByCode(@JdbcConn Connection con,@Nullable Boolean active,@Nullable String code)throws SQLException {
        return this.collectDeviceDao.queryByCode(con, active, code == null ? null : "%" + code + "%");
    }

    @Get
    @Path("/device/checkCode")
    public CollectDevice deviceByCode(@JdbcConn Connection con,String code,@Nullable String id,@Nullable Boolean active)throws SQLException {
        return this.collectDeviceDao.queryByCode(con, code,id,active);
    }

    @Get
    @Path("/device/checkSeq")
    public CollectDevice serverDevice(@JdbcConn Connection con,String serverId,int seq,@Nullable String id,@Nullable Boolean active) throws SQLException {
        return this.collectDeviceDao.queryBySeqAndServer(con, serverId, seq,id,active);
    }

    @Post
    @Path("/transducer")
    public String insert(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, Transducer transducer) throws SQLException {
        String id = StringUtil.buildUUID();
        transducer.setId(id);
        transducer.setActive(true);
        transducer.setCreator(sessionUser.getId());
        transducer.setModifier(sessionUser.getId());
        CollectDevice device = this.collectDeviceDao.query(con, transducer.getDeviceId());
        transducer.setServerId(device.getServerId());
        this.transducerDao.insert(con, transducer);
        return id;
    }

    @Get
    @Path("/transducer/qo")
    public Transducer queryTransducer(@JdbcConn Connection con, String id) throws SQLException {
        return this.transducerDao.query(con, id);
    }

    @Get
    @Path("/transducer/pq")
    public PageQueryResult<Transducer> pageQueryTransducer(@JdbcConn Connection con, @Nullable Boolean active, @Nullable String cdCode, @Nullable String code, int pageSize, int pageNo) throws SQLException {
        return this.transducerDao.pageQuery(con, active, cdCode, code, pageSize, pageNo);
    }

    @Post
    @Path("/transducer/update")
    public void updateTransducer(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, Transducer transducer) throws SQLException {
        transducer.setModifier(sessionUser.getId());
        CollectDevice device = this.collectDeviceDao.query(con, transducer.getDeviceId());
        transducer.setServerId(device.getServerId());
        this.transducerDao.update(con, transducer);
    }

    @Get
    @Path("/transducer/delete")
    public void deleteTransducer(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, String id) throws SQLException {
        this.transducerDao.logicDelete(con, sessionUser.getId(), id);
    }

    @Get
    @Path("/transducer/byDevice")
    public List<Transducer> queryByDevcie(@JdbcConn Connection con,@Nullable Boolean active, String id)throws SQLException {
        return this.transducerDao.queryByDevice(con,active,id);
    }

    @Get
    @Path("/transducer/checkCode")
    public Transducer qoByCode(@JdbcConn Connection con,String code,@Nullable String id,@Nullable Boolean active)throws SQLException{
        return this.transducerDao.queryByCode(con, code,id,active);
    }

    @Get
    @Path("/transducer/checkSeq")
    public Transducer deviceTransducer(@JdbcConn Connection con,String deviceId,int seq,@Nullable String id,@Nullable Boolean active)throws SQLException {
        return this.transducerDao.queryBySeqAndDeviceFromServer(con, deviceId, seq,id,active);
    }

    @Get
    @Path("/all/byUser")
    public Map<String, Object> queryAll(@JdbcConn Connection con, @Nullable Boolean active, @LoginUser SessionUser user) throws SQLException {
        Map<String, Object> map = new HashMap<>();
        List<Bridge> bridges = this.bridgeDao.listQueryByUser(con, active, user.getId());
        map.put("bridge", bridges);
        if (bridges.size() > 0) {
            String[] bridgeId = new String[bridges.size()];
            for (int i = 0; i < bridges.size(); ++i) {
                bridgeId[i] = bridges.get(i).getId();
            }
            List<BridgeServer> bridgeServers = this.bridgeServerDao.queryByBridges(con, active, bridgeId);
            map.put("server", bridgeServers);
            if (bridgeServers.size() > 0) {
                String[] serverId = new String[bridgeServers.size()];
                for (int i = 0; i < bridgeServers.size(); ++i) {
                    serverId[i] = bridgeServers.get(i).getId();
                }
                List<CollectDevice> collectDevices = this.collectDeviceDao.queryByServers(con, active, serverId);
                map.put("device", collectDevices);
                if (collectDevices.size() > 0) {
                    String[] deviceId = new String[collectDevices.size()];
                    for (int i = 0; i < collectDevices.size(); ++i) {
                        deviceId[i] = collectDevices.get(i).getId();
                    }
                    List<Transducer> transducers = this.transducerDao.queryByUser(con, active, deviceId);
                    map.put("transducer", transducers);
                } else {
                    map.put("transducer", collectDevices);
                }
            } else {
                map.put("device", bridgeServers);
                map.put("transducer", bridgeServers);
            }
        } else {
            map.put("server", bridges);
            map.put("device", bridges);
            map.put("transducer", bridges);
        }
        return map;
    }



    private void remove(List<UploadFile> list) {
        for (UploadFile file : list) {
            if (file.fn != null)
                file.fn.delete();
        }
    }

    private UploadFile buildTargetFile(String ext) {
        StringBuilder sb = new StringBuilder();
        sb.append(System.currentTimeMillis() / (1000 * 60 * 60 * 24));
        File path = new File(this.imgPath, sb.toString());
        if (!path.exists()) {
            synchronized (this) {
                if (!path.mkdirs())
                    throw new RuntimeException("mkdir error[" + path + "]");
                FN_IDX.set(1);
            }
        }
        File file;
        do {
            String fn = FN_IDX.toString();
            if (ext.isEmpty()) {
                file = new File(path, fn);
            } else {
                file = new File(path, fn + "." + ext);
            }
            FN_IDX.incrementAndGet();
        } while (file.exists());
        sb.append("/").append(file.getName());
        UploadFile uf = new UploadFile();
        uf.setFn(file);
        uf.setUri("/" + sb.toString());
        return uf;
    }

    private String normalizeFileName(String fn) {
        int index = fn.indexOf('\\');
        if (index >= 0) {
            fn = fn.substring(fn.lastIndexOf('\\') + 1);
        }
        index = fn.indexOf('/');
        if (index >= 0) {
            fn = fn.substring(fn.lastIndexOf('/') + 1);
        }
        if (fn.length() == 0)
            throw new RuntimeException("invalid filename in Multipart/data request");
        return fn;
    }

    public static class UploadFile {
        private String name;
        private String uri;
        private long size;
        private transient File fn;

        public File getFn() {
            return fn;
        }

        public void setFn(File fn) {
            this.fn = fn;
        }

        public String getName() {
            return name;
        }

        public void setName(String name) {
            this.name = name;
        }

        public String getUri() {
            return uri;
        }

        public void setUri(String uri) {
            this.uri = uri;
        }

        public long getSize() {
            return size;
        }

        public void setSize(long size) {
            this.size = size;
        }
    }

}
